import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import { level } from '../level.enum';
import loggerOptions from '../logger.options';
import {
  AD_CREATED_CRIT_MESSAGE_HANDLER,
  AD_CREATED_INFO_MESSAGE_HANDLER,
  AD_CREATED_WARNING_MESSAGE_HANDLER,
  AD_DELETED_CRIT_MESSAGE_HANDLER,
  AD_DELETED_INFO_MESSAGE_HANDLER,
  AD_HEALTH_CRIT_MESSAGE_HANDLER,
  AD_READ_WARNING_MESSAGE_HANDLER,
  AD_UPDATED_CRIT_MESSAGE_HANDLER,
  AD_UPDATED_INFO_MESSAGE_HANDLER,
} from '@src/app.constants';

export class AdMessageHandler {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    name: AD_CREATED_INFO_MESSAGE_HANDLER,
  })
  public async adCreatedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('ad', level.info, 'info', 'created'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_CREATED_WARNING_MESSAGE_HANDLER,
  })
  public async adCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('ad', level.warning, 'warning', 'created'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_CREATED_CRIT_MESSAGE_HANDLER,
  })
  public async adCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('ad', level.crit, 'critical', 'created'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_DELETED_INFO_MESSAGE_HANDLER,
  })
  public async adDeletedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('ad', level.info, 'info', 'deleted'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_DELETED_CRIT_MESSAGE_HANDLER,
  })
  public async adDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('ad', level.crit, 'critical', 'deleted'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_READ_WARNING_MESSAGE_HANDLER,
  })
  public async adReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('ad', level.warning, 'warning', 'read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_UPDATED_INFO_MESSAGE_HANDLER,
  })
  public async adUpdatedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('ad', level.info, 'info', 'updated'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_UPDATED_CRIT_MESSAGE_HANDLER,
  })
  public async adUpdatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('ad', level.crit, 'critical', 'updated'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: AD_HEALTH_CRIT_MESSAGE_HANDLER,
  })
  public async adHealthCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('ad', level.crit, 'critical', 'health'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
