import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import loggerOptions from '../logger.options';
import { level } from '../level.enum';
import {
  TERRITORY_CREATED_CRIT_MESSAGE_HANDLER,
  TERRITORY_CREATED_INFO_MESSAGE_HANDLER,
  TERRITORY_CREATED_WARNING_MESSAGE_HANDLER,
  TERRITORY_DELETED_CRIT_MESSAGE_HANDLER,
  TERRITORY_DELETED_INFO_MESSAGE_HANDLER,
  TERRITORY_HEALTH_CRIT_MESSAGE_HANDLER,
  TERRITORY_READ_WARNING_MESSAGE_HANDLER,
  TERRITORY_UPDATED_CRIT_MESSAGE_HANDLER,
  TERRITORY_UPDATED_INFO_MESSAGE_HANDLER,
} from '@src/app.constants';

export class TerritoryMessageHandler {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    name: TERRITORY_CREATED_INFO_MESSAGE_HANDLER,
  })
  public async territoryCreatedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.info, 'info', 'created'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_CREATED_WARNING_MESSAGE_HANDLER,
  })
  public async territoryCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.warning, 'warning', 'created'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_CREATED_CRIT_MESSAGE_HANDLER,
  })
  public async territoryCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.crit, 'critical', 'created'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_DELETED_INFO_MESSAGE_HANDLER,
  })
  public async territoryDeletedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.info, 'info', 'deleted'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_DELETED_CRIT_MESSAGE_HANDLER,
  })
  public async territoryDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.crit, 'critical', 'deleted'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_READ_WARNING_MESSAGE_HANDLER,
  })
  public async territoryReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.warning, 'warning', 'read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_UPDATED_INFO_MESSAGE_HANDLER,
  })
  public async territoryUpdatedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.info, 'info', 'updated'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_UPDATED_CRIT_MESSAGE_HANDLER,
  })
  public async territoryUpdatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.crit, 'critical', 'updated'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: TERRITORY_HEALTH_CRIT_MESSAGE_HANDLER,
  })
  public async territoryHealthCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('territory', level.crit, 'critical', 'health'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
