import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import loggerOptions from '../logger.options';
import { level } from '../level.enum';
import {
  USER_CREATED_CRIT_MESSAGE_HANDLER,
  USER_CREATED_INFO_MESSAGE_HANDLER,
  USER_CREATED_WARNING_MESSAGE_HANDLER,
  USER_DELETED_CRIT_MESSAGE_HANDLER,
  USER_DELETED_INFO_MESSAGE_HANDLER,
  USER_HEALTH_CRIT_MESSAGE_HANDLER,
  USER_READ_WARNING_MESSAGE_HANDLER,
  USER_UPDATED_CRIT_MESSAGE_HANDLER,
  USER_UPDATED_INFO_MESSAGE_HANDLER,
} from '@src/app.constants';

export class UserMessageHandler {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    name: USER_CREATED_INFO_MESSAGE_HANDLER,
  })
  public async userCreatedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('user', level.info, 'info', 'created'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_CREATED_WARNING_MESSAGE_HANDLER,
  })
  public async userCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.warning, 'warning', 'created'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_CREATED_CRIT_MESSAGE_HANDLER,
  })
  public async userCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'created'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_DELETED_INFO_MESSAGE_HANDLER,
  })
  public async userDeletedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('user', level.info, 'info', 'deleted'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_DELETED_CRIT_MESSAGE_HANDLER,
  })
  public async userDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'deleted'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_READ_WARNING_MESSAGE_HANDLER,
  })
  public async userReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.warning, 'warning', 'read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_UPDATED_INFO_MESSAGE_HANDLER,
  })
  public async userUpdatedInfoHandler(message: string) {
    this.logger.configure(loggerOptions('user', level.info, 'info', 'updated'));
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_UPDATED_CRIT_MESSAGE_HANDLER,
  })
  public async userUpdatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'updated'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: USER_HEALTH_CRIT_MESSAGE_HANDLER,
  })
  public async userHealthCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('user', level.crit, 'critical', 'health'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
